import React, { Fragment } from "react";
import { Link } from "@components/Link";

interface BaseProps {
  type: "primary" | "secondary" | "tertiary";
  size?: "xs" | "sm" | "md" | "lg" | "xl";
  className?: string;
}

type NormalOrHref =
  | {
      text?: string;
      href?: never;
      icon?: React.ReactNode;
      iconRight?: boolean;
      onClick?: () => void;
      disabled?: boolean;
      buttonType?: "button" | "submit" | "reset";
      newTab?: never;
    }
  | {
      text?: string;
      href: string;
      icon?: React.ReactNode;
      iconRight?: boolean;
      onClick?: never;
      disabled?: never;
      buttonType?: never;
      newTab?: boolean;
    };

type CircularOrRounded =
  | {
      circular?: true;
      icon?: React.ReactNode;
      rounded?: never;
      text?: never;
    }
  | {
      circular?: never;
      icon?: React.ReactNode;
      rounded?: true;
      text?: string;
    };

export type ButtonProps = BaseProps & NormalOrHref & CircularOrRounded;

export const Button: React.FC<ButtonProps> = ({
  text,
  type = "primary",
  size,
  rounded,
  circular,
  disabled,
  icon,
  iconRight,
  onClick,
  href,
  buttonType = "button",
  className = "",
  newTab,
}) => {
  const STYLE_MAPPING: Record<string, string> = {
    primary:
      "bg-grey-900 hover:bg-grey-700 focus:bg-grey-700 text-white hover:text-grey-400 focus:text-grey-400 no-underline not-prose",
    secondary:
      "bg-grey-400 hover:bg-grey-200 focus:bg-grey-200 text-grey-900 hover:text-grey-500 focus:text-grey-500 no-underline not-prose",
    tertiary:
      "bg-white hover:bg-grey-100 focus:bg-grey-100 text-grey-900 hover:text-grey-500 focus:text-grey-500 no-underline not-prose",
  };

  const SIZE_MAPPING: Record<string, string> = {
    xs: "px-3 py-2",
    sm: "px-3.5 py-2.5",
    md: "px-4 py-3",
    lg: "px-5 py-3.5",
    xl: "px-6 py-4",
    default:
      "px-3 sm:px-3.5 md:px-4 lg:px-5 xl:px-6 py-2 sm:py-2.5 md:py-3 lg:py-3.5 xl:py-4",
  };

  const TEXT_MAPPING: Record<string, string> = {
    xs: "font-plex font-medium text-xs leading-4",
    sm: "font-plex font-medium text-sm leading-4",
    md: "font-plex font-medium text-sm leading-5",
    lg: "font-plex font-medium text-base leading-6",
    xl: "font-plex font-medium text-base leading-6",
    default:
      "font-plex text-xs sm:text-sm lg:text-base leading-4 md:leading-5 lg:leading-6 font-medium",
  };

  const CIRCULAR_SIZE_MAPPING: Record<string, string> = {
    xs: "w-7 h-7",
    sm: "w-8 h-8",
    md: "w-9 h-9",
    lg: "w-10 h-10",
    xl: "w-11 h-11",
    default:
      "w-7 sm:w-8 md:w-9 lg:w-10 xl:w-11 h-7 sm:h-8 md:h-9 lg:h-10 xl:h-11",
  };

  const CIRCULAR_STYLE_MAPPING: Record<string, string> = {
    primary:
      "flex items-center justify-center rounded-full bg-grey-900 hover:bg-grey-700 focus:bg-grey-700 text-white hover:text-grey-400 focus:text-grey-400",
    secondary:
      "flex items-center justify-center rounded-full bg-grey-400 hover:bg-grey-200 focus:bg-grey-200 text-grey-900 hover:text-grey-500 focus:text-grey-500",
    tertiary:
      "flex items-center justify-center rounded-full bg-white hover:bg-grey-100 focus:bg-grey-100 text-grey-900 hover:text-grey-500 focus:text-grey-500",
  };

  const ADDITIONAL_STYLE_MAPPING: Record<string, string> = {
    rounded: "rounded-full",
    square: "rounded",
    outline:
      "sm:focus:ring-2 focus:ring-offset-2 focus:ring-offset-white focus:ring-grey-900",
    transition: "transition-all ease-in-out duration-200",
    disabled: "opacity-50 pointer-events-none",
  };

  const renderButtonContent = () => {
    return (
      <Fragment>
        {icon && (
          <span className={`text-center ${iconRight ? "order-2 ml-2.5" : "order-1 mr-2.5"}`}>
            {icon}
          </span>
        )}
        <span className={`text-center ${iconRight ? "order-1" : "order-2"}`}>{text}</span>
      </Fragment>
    );
  };

  return (
    <Fragment>
      {!circular ? (
        <Fragment>
          {href ? (
            <Link
              to={href}
              className={`flex items-center justify-center w-fit ${
                ADDITIONAL_STYLE_MAPPING["outline"]
              } ${
                rounded
                  ? ADDITIONAL_STYLE_MAPPING["rounded"]
                  : ADDITIONAL_STYLE_MAPPING["square"]
              } ${ADDITIONAL_STYLE_MAPPING["transition"]} ${
                size ? TEXT_MAPPING[size] : TEXT_MAPPING["default"]
              } ${size ? SIZE_MAPPING[size] : SIZE_MAPPING["default"]} ${
                STYLE_MAPPING[type]
              } ${className}`}
              {...(newTab ? { target: "_blank", rel: "noopener" } : {})}
            >
              {renderButtonContent()}
            </Link>
          ) : (
            <button
              className={`flex items-center justify-center ${
                ADDITIONAL_STYLE_MAPPING["outline"]
              } ${
                rounded
                  ? ADDITIONAL_STYLE_MAPPING["rounded"]
                  : ADDITIONAL_STYLE_MAPPING["square"]
              } ${ADDITIONAL_STYLE_MAPPING["transition"]} ${
                size ? TEXT_MAPPING[size] : TEXT_MAPPING["default"]
              } ${size ? SIZE_MAPPING[size] : SIZE_MAPPING["default"]} ${
                STYLE_MAPPING[type]
              } ${
                disabled ? ADDITIONAL_STYLE_MAPPING["disabled"] : ""
              } ${className}`}
              disabled={Boolean(disabled)}
              onClick={onClick}
              type={buttonType}
            >
              {renderButtonContent()}
            </button>
          )}
        </Fragment>
      ) : (
        <Fragment>
          {href ? (
            <Link
              to={href}
              className={`${ADDITIONAL_STYLE_MAPPING["outline"]} ${
                ADDITIONAL_STYLE_MAPPING["transition"]
              } ${
                size
                  ? CIRCULAR_SIZE_MAPPING[size]
                  : CIRCULAR_SIZE_MAPPING["default"]
              } ${CIRCULAR_STYLE_MAPPING[type]} ${className}`}
              {...(newTab ? { target: "_blank", rel: "noopener" } : {})}
            >
              {icon}
            </Link>
          ) : (
            <button
              className={`${ADDITIONAL_STYLE_MAPPING["outline"]} ${
                ADDITIONAL_STYLE_MAPPING["transition"]
              } ${
                size
                  ? CIRCULAR_SIZE_MAPPING[size]
                  : CIRCULAR_SIZE_MAPPING["default"]
              } ${CIRCULAR_STYLE_MAPPING[type]} ${
                disabled ? ADDITIONAL_STYLE_MAPPING["disabled"] : ""
              } ${className}`}
              disabled={Boolean(disabled)}
              onClick={onClick}
              type={buttonType}
            >
              {icon}
            </button>
          )}
        </Fragment>
      )}
    </Fragment>
  );
};
