import React, {
  useState,
  useEffect,
  Children,
  isValidElement,
  Fragment
} from "react";
import { CommonUtils } from "@utils/CommonUtils";
import { ITabItem } from "@interfaces/ITab";

type BaseProps = {
  className?: string;
};

type ChildrenProps = {
  children?: React.ReactNode;
  entries?: never;
};

type EntriesProps = {
  children?: never;
  entries?: ITabItem[];
};

export type TabsProps = BaseProps & (ChildrenProps | EntriesProps);

interface ITabContent {
  id: string;
  label: string;
  content: string;
}

export const Tabs: React.FC<TabsProps> = ({ children, entries, className }) => {
  const [activeTab, setActiveTab] = useState<string | null>(null);
  const [tabData, setTabData] = useState<ITabContent[]>([]);

  useEffect(() => {
    if (children) {
      const data: ITabContent[] = [];
      Children.forEach(children, (child) => {
        if (!isValidElement(child)) return;
        data.push({
          id: CommonUtils.generateID(),
          label: child.props.label,
          content: child.props.children,
        });
      });
      setTabData(data);
      setActiveTab(data[0].id);
    }
  }, [children]);

  useEffect(() => {
    if (entries) {
      const data: ITabContent[] = entries.map((entry) => {
        return {
          id: CommonUtils.generateID(),
          label: entry.label,
          content: entry.content,
        };
      });
      setTabData(data);
      setActiveTab(data[0].id);
    }
  }, [entries]);

  return (
    <div className={`w-full not-prose ${className}`}>
      <ol className="flex overflow-x-auto overflow-y-hidden">
        {tabData.map((tab) => (
          <TabPanel
            key={tab.id}
            label={tab.label}
            active={tab.id === activeTab}
            onClick={() => setActiveTab(tab.id)}
          />
        ))}
      </ol>
      <div className="py-4 xl:py-6">
        {tabData.map((tab) => {
          if (activeTab === tab.id) {
            return <TabContent key={tab.id} content={tab.content} />;
          }
        })}
      </div>
    </div>
  );
};

interface TabPanelProps {
  label: string;
  active: boolean;
  onClick: () => void;
}

const TabPanel: React.FC<TabPanelProps> = ({ label, active, onClick }) => {
  const beforeClasses =
    "before:block before:content-[attr(data-label)] before:font-semibold before:h-0 before:overflow-hidden before:invisible";
  const afterClasses =
    "after:content-[''] after:absolute after:left-0 after:-bottom-0.5 after:bg-grey-transparent hover:after:bg-grey-500 after:scale-x-0 hover:after:scale-x-100 after:transition-all after:duration-300 after:ease-in-out after:origin-left after:w-full after:h-0.5";
  return (
    <li
      className={`group relative inline-block text-[0px] px-4 xl:px-5 py-2 xl:py-3 cursor-pointer select-none border-b-2 border-transparent ${afterClasses} ${
        active
          ? "after:scale-x-100 after:bg-grey-900 hover:after:bg-grey-900"
          : ""
      }`}
      onClick={onClick}
    >
      <span
        className={`xl:group-hover:font-semibold inline-block whitespace-nowrap font-plex text-center text-base xl:text-lg leading-6 ${beforeClasses} ${
          active
            ? "font-semibold text-grey-900 text-lg"
            : "font-medium xl:font-normal text-grey-500"
        }`}
        data-label={label}
      >
        {label}
      </span>
    </li>
  );
};

interface TabContentProps {
  content: string;
  prose?: boolean;
}

const TabContent: React.FC<TabContentProps> = ({ content }) => {
  return <div dangerouslySetInnerHTML={{ __html: content }}></div>;
};

interface TabProps {
  children: React.ReactNode;
  label: string;
}

export const Tab: React.FC<TabProps> = ({ children }) => {
  return <Fragment>{children}</Fragment>;
};
